@@ -1,11 +1,8 @@
 import logging
 import os
 from abc import abstractmethod
-from copy import deepcopy
-from typing import Any, List, Mapping, Union, cast
+from typing import TYPE_CHECKING, Any, List, Mapping, TypeVar, Union, cast  # noqa: F401
 
-import lightning as L
-import numpy as np
 import torch
 from lightning.fabric.utilities.rank_zero import rank_zero_only
 from omegaconf import DictConfig
@@ -22,10 +19,13 @@ from fusion_bench.models.wrappers.layer_wise_fusion import (
     get_layer_wise_weights,
 )
 from fusion_bench.utils.data import load_tensor_from_file
-from fusion_bench.utils.state_dict_arithmetic import state_dict_sub
-from fusion_bench.utils.type import StateDictType
+from fusion_bench.utils.type import TorchModelType
 
 from .entropy_loss import entropy_loss
+from .utils import get_memory_usage
+
+if TYPE_CHECKING:
+    from fusion_bench.programs.fabric_fusion_program import FabricModelFusionProgram
 
 log = logging.getLogger(__name__)
 
@@ -35,11 +35,27 @@ class LayerWiseAdaMergingAlgorithm(
     LightningFabricMixin,
     SimpleProfilerMixin,
 ):
+    _program: "FabricModelFusionProgram"
+    """The program that this algorithm is running on."""
+
+    """
+    Implements the Layer-Wise AdaMerging Algorithm.
+
+    This class merges the layers of a pretrained model with those of several fine-tuned models.
+    The merging is controlled by layer-wise weights, which can be initialized based on a provided configuration or loaded from a file.
+    """
+
     def __init__(self, algorithm_config: DictConfig):
+        """
+        Initialize the LayerWiseAdaMergingAlgorithm with the given configuration.
+
+        Args:
+            algorithm_config (DictConfig): The configuration for the algorithm.
+        """
         super().__init__(algorithm_config)
 
     @torch.no_grad()
-    def construct_layer_wise_merged_model(self, modelpool: ModelPool):
+    def construct_layer_wise_merged_model(self, modelpool: "ModelPool"):
         """
         Constructs a wrapped layer-wise merged model from model pool.
 
@@ -89,6 +105,13 @@ class LayerWiseAdaMergingAlgorithm(
 
     @rank_zero_only
     def save_merging_weights(self, file_path: str, merging_weights: torch.Tensor):
+        """
+        Save the merging weights to a file.
+
+        Args:
+            file_path (str): The path to save the merging weights.
+            merging_weights (torch.Tensor): The merging weights to save.
+        """
         if self.fabric.is_global_zero and self.config.get(
             "save_merging_weights", False
         ):
@@ -102,7 +125,18 @@ class LayerWiseAdaMergingAlgorithm(
                 os.makedirs(os.path.dirname(save_path), exist_ok=True)
             torch.save(merging_weights.detach().cpu(), save_path)
 
-    def run(self, modelpool: ModelPool):
+    def run(self, modelpool: ModelPool, **kwargs):
+        """
+        Run the Layer-Wise AdaMerging Algorithm.
+
+        This method constructs the wrapped model and performs test-time adaptation if necessary.
+
+        Args:
+            modelpool (ModelPool): The model pool containing the pretrained and fine-tuned models.
+
+        Returns:
+            LayerWiseMergedModel: The merged model after test-time adaptation.
+        """
         log.info("Fusing models using layer-wise adaptive merging.")
         self.modelpool = modelpool
         self.log_hyperparams(self.config)
@@ -132,16 +166,43 @@ class LayerWiseAdaMergingAlgorithm(
     def get_shuffled_test_loader_iter(self, task: str) -> DataLoader:
         """
         Loader of test dataset for test-time adaptation. labels are not needed.
+
+        Args:
+            task (str): The name of the task.
+
+        Returns:
+            DataLoader: The data loader for the test dataset.
         """
         pass
 
     @abstractmethod
     def compute_logits(self, module, images: Tensor, task: str) -> Tensor:
+        """
+        Compute the logits for the given images and task.
+
+        Args:
+            module: The model module.
+            images (Tensor): The input images.
+            task (str): The name of the task.
+
+        Returns:
+            Tensor: The computed logits.
+        """
         pass
 
-    def test_time_adaptation(self, module: LayerWiseMergedModel):
+    def test_time_adaptation(self, module: "LayerWiseMergedModel[TorchModelType]"):
+        """
+        Perform test-time adaptation on the merged model.
+
+        This method adapts the merging weights during test-time to improve performance.
+
+        Args:
+            module (LayerWiseMergedModel): The merged model.
+
+        Returns:
+            LayerWiseMergedModel: The adapted merged model.
+        """
         self.on_test_time_adaptation_start()
-        config = self.config
 
         # configure optimizer
         if self.config.optimizer == "adam":
@@ -186,5 +247,6 @@ class LayerWiseAdaMergingAlgorithm(
             self.fabric.log_dict(metrics, step=step_idx)
             pbar.set_postfix(metrics)
 
+        log.info(get_memory_usage(f"after adamerging, the memory usage of GPU is:"))
         self.print_profile_summary()
         return module
