@@ -5,7 +5,7 @@ Examples:
 
 ```bash
 fusion_bench \
-    fabric_logger.name=ViT-B-32/clip_concrete_task_wise_adamerging \
+    fabric.loggers.name=ViT-B-32/clip_concrete_task_wise_adamerging \
     method=clip_concrete_task_wise_adamerging \
     modelpool=clip-vit-base-patch32_TA8 \
     taskpool=clip-vit-classification_TA8
@@ -13,7 +13,7 @@ fusion_bench \
 
 ```bash
 fusion_bench \
-    fabric_logger.name=ViT-B-32/clip_concrete_layer_wise_adamerging \
+    fabric.loggers.name=ViT-B-32/clip_concrete_layer_wise_adamerging \
     method=clip_concrete_layer_wise_adamerging \
     modelpool=clip-vit-base-patch32_TA8 \
     taskpool=clip-vit-classification_TA8
@@ -27,11 +27,14 @@ from copy import deepcopy
 import torch
 from tqdm.autonotebook import tqdm
 
-from fusion_bench.method import ModelFusionAlgorithm
+from fusion_bench.compat.method import ModelFusionAlgorithm
+from fusion_bench.compat.modelpool import to_modelpool
+from fusion_bench.compat.modelpool.huggingface_clip_vision import (
+    HuggingFaceClipVisionPool,
+)
 from fusion_bench.method.adamerging.entropy_loss import entropy_loss
+from fusion_bench.mixins import CLIPClassificationMixin
 from fusion_bench.mixins.simple_profiler import SimpleProfilerMixin
-from fusion_bench.modelpool import ModelPool, to_modelpool
-from fusion_bench.modelpool.huggingface_clip_vision import HuggingFaceClipVisionPool
 from fusion_bench.models.masks import MaskModel, mask_sparsity
 from fusion_bench.models.wrappers.layer_wise_fusion import (
     LayerWiseMergedModel,
@@ -41,10 +44,8 @@ from fusion_bench.models.wrappers.task_wise_fusion import (
     TaskWiseMergedModel,
     get_task_wise_weights,
 )
-from fusion_bench.tasks.clip_classification.clip_mixin import CLIPClassificationMixin
 from fusion_bench.utils.dtype import parse_dtype
 from fusion_bench.utils.parameters import print_parameters
-from fusion_bench.utils.type import StateDictType
 
 log = logging.getLogger(__name__)
 
