@@ -1,9 +1,8 @@
 import logging
-from typing import List, Optional, Union, cast
+from typing import List, Optional, Union, cast  # noqa: F401
 
 import torch
 from omegaconf import open_dict
-from torch import nn
 from transformers import (
     LlamaForCausalLM,
     LlamaModel,
@@ -32,6 +31,14 @@ def _substitute_experts(
     expert_model: Union[LlamaModel, MistralModel],
     mixtral_model: MixtralModel,
 ):
+    """
+    Substitute the experts of the `MixtralModel` with the models from the modelpool.
+
+    Args:
+        expert_idx (int): The index of the expert to substitute.
+        expert_model (Union[LlamaModel, MistralModel]): The expert model to substitute.
+        mixtral_model (MixtralModel): The MixtralModel to substitute the experts in.
+    """
     for input_layer, output_layer in zip(expert_model.layers, mixtral_model.layers):
         output_layer = cast(MixtralDecoderLayer, output_layer)
         input_layer = cast(Union[LlamaDecoderLayer, MistralDecoderLayer], input_layer)
@@ -71,6 +78,10 @@ class MixtralMoEMergingAlgorithm(MixtralUpscalingAlgorithm):
 
 
 class MixtralForCausalLMMergingAlgorithm(MixtralForCausalLMUpscalingAlgorithm):
+    """
+    This class is responsible for merging models into a `MixtralForCausalLM`.
+    """
+
     @torch.no_grad()
     def run(self, modelpool: BaseModelPool) -> MixtralForCausalLM:
         """
