@@ -1,12 +1,11 @@
 import logging
 import random
-from copy import deepcopy
-from typing import List, Mapping, Union
+from typing import List, Mapping, Union  # noqa: F401
 
 import torch
-from torch import Tensor, nn
+from torch import nn
 
-from fusion_bench.method import BaseModelFusionAlgorithm
+from fusion_bench.method import BaseAlgorithm
 from fusion_bench.modelpool import BaseModelPool
 
 log = logging.getLogger(__name__)
@@ -53,11 +52,19 @@ def recombine_state_dict(models: List[nn.Module]):
     return models
 
 
-class ModelRecombinationAlgorithm(BaseModelFusionAlgorithm):
+class ModelRecombinationAlgorithm(BaseAlgorithm):
     """
     Model recombination recombinates the layers of the given models, to create a new set of models.
     """
 
+    _config_mapping = BaseAlgorithm._config_mapping | {
+        "return_modelpool": "return_modelpool",
+    }
+
+    def __init__(self, return_modelpool: bool, **kwargs):
+        self.return_modelpool = return_modelpool
+        super().__init__(**kwargs)
+
     @torch.no_grad()
     def run(
         self,
