@@ -3,14 +3,15 @@ from copy import deepcopy
 from typing import Dict, List, Mapping, Optional, Union
 
 import torch
-from torch import Tensor, nn
+from torch import nn
 
-from fusion_bench.method.base_algorithm import BaseModelFusionAlgorithm
+from fusion_bench.method.base_algorithm import BaseAlgorithm
 from fusion_bench.mixins.simple_profiler import SimpleProfilerMixin
 from fusion_bench.modelpool import BaseModelPool
 from fusion_bench.utils.state_dict_arithmetic import (
     state_dict_add,
     state_dict_avg,
+    state_dict_div,
     state_dict_mul,
 )
 from fusion_bench.utils.type import StateDictType
@@ -18,14 +19,18 @@ from fusion_bench.utils.type import StateDictType
 log = logging.getLogger(__name__)
 
 
-def simple_average(modules: List[Union[nn.Module, StateDictType]]):
-    """
+def simple_average(
+    modules: List[Union[nn.Module, StateDictType]],
+    base_module: Optional[nn.Module] = None,
+):
+    R"""
     Averages the parameters of a list of PyTorch modules or state dictionaries.
 
     This function takes a list of PyTorch modules or state dictionaries and returns a new module with the averaged parameters, or a new state dictionary with the averaged parameters.
 
     Args:
         modules (List[Union[nn.Module, StateDictType]]): A list of PyTorch modules or state dictionaries.
+        base_module (Optional[nn.Module]): A base module to use for the new module. If provided, the averaged parameters will be loaded into this module. If not provided, a new module will be created by copying the first module in the list.
 
     Returns:
         module_or_state_dict (Union[nn.Module, StateDictType]): A new PyTorch module with the averaged parameters, or a new state dictionary with the averaged parameters.
@@ -34,14 +39,18 @@ def simple_average(modules: List[Union[nn.Module, StateDictType]]):
         >>> import torch.nn as nn
         >>> model1 = nn.Linear(10, 10)
         >>> model2 = nn.Linear(10, 10)
-        >>> averaged_model = simple_averageing([model1, model2])
+        >>> averaged_model = simple_average([model1, model2])
 
         >>> state_dict1 = model1.state_dict()
         >>> state_dict2 = model2.state_dict()
-        >>> averaged_state_dict = simple_averageing([state_dict1, state_dict2])
+        >>> averaged_state_dict = simple_average([state_dict1, state_dict2])
     """
+    assert len(modules) > 0, "modules must be a non-empty list"
     if isinstance(modules[0], nn.Module):
-        new_module = deepcopy(modules[0])
+        if base_module is None:
+            new_module = deepcopy(modules[0])
+        else:
+            new_module = base_module
         state_dict = state_dict_avg([module.state_dict() for module in modules])
         new_module.load_state_dict(state_dict)
         return new_module
@@ -50,7 +59,7 @@ def simple_average(modules: List[Union[nn.Module, StateDictType]]):
 
 
 class SimpleAverageAlgorithm(
-    BaseModelFusionAlgorithm,
+    BaseAlgorithm,
     SimpleProfilerMixin,
 ):
     @torch.no_grad()
@@ -85,12 +94,15 @@ class SimpleAverageAlgorithm(
                 print(f"load model of type: {type(model).__name__}")
             with self.profile("merge weights"):
                 if sd is None:
+                    # Initialize the state dictionary with the first model's state dictionary
                     sd = model.state_dict(keep_vars=True)
                     forward_model = model
                 else:
+                    # Add the current model's state dictionary to the accumulated state dictionary
                     sd = state_dict_add(sd, model.state_dict(keep_vars=True))
         with self.profile("merge weights"):
-            sd = state_dict_mul(sd, 1 / len(modelpool.model_names))
+            # Divide the accumulated state dictionary by the number of models to get the average
+            sd = state_dict_div(sd, len(modelpool.model_names))
 
         forward_model.load_state_dict(sd)
         # print profile report and log the merged models
