@@ -1,21 +1,47 @@
+import functools
 import logging
 import os
-from typing import TYPE_CHECKING, Any, Optional, TypeVar
+from typing import TYPE_CHECKING, Any, List, Optional, TypeVar
 
 import lightning as L
 import torch
+from lightning.fabric.connector import _is_using_cli
 from lightning.fabric.loggers import TensorBoardLogger
 from lightning.fabric.utilities.rank_zero import rank_zero_only
 from omegaconf import DictConfig, OmegaConf
 
+from fusion_bench.utils import import_object
+from fusion_bench.utils.instantiate_utils import instantiate
+
 if TYPE_CHECKING:
     import lightning.fabric.loggers.tensorboard
+    from lightning.fabric.strategies import FSDPStrategy
 
 log = logging.getLogger(__name__)
 
 TensorOrModule = TypeVar("TensorOrModule", torch.Tensor, torch.nn.Module, Any)
 
 
+def get_policy(*args: str) -> set:
+    """
+    Get the policy from the provided list of policy names.
+
+    Args:
+        *args (str): A list of policy names.
+
+    Returns:
+        set: A set of policy objects.
+    """
+    return {import_object(arg) for arg in args}
+
+
+def get_size_based_auto_wrap_policy(*args, **kwargs):
+    from torch.distributed.fsdp.wrap import size_based_auto_wrap_policy
+
+    policy = functools.partial(size_based_auto_wrap_policy, *args, **kwargs)
+    return policy
+
+
 class LightningFabricMixin:
     """
     A mixin class for integrating Lightning Fabric into a project.
@@ -46,20 +72,16 @@ class LightningFabricMixin:
 
         Expected configuration keys:
         - fabric: The configuration for the Lightning Fabric.
-        - fabric_logger: The configuration for the TensorBoardLogger.
+        - fabric.loggers: The configuration for the TensorBoardLogger.
         """
         if self._fabric_instance is None:
             if config.get("fabric", None) is None:
                 log.warning("No fabric configuration found. use default settings.")
                 self._fabric_instance = L.Fabric()
             else:
-                if config.get("fabric_logger", None) is not None:
-                    logger = TensorBoardLogger(**config.fabric_logger)
-                else:
-                    logger = None
-                log.info("Launching Lightning Fabric")
-                self._fabric_instance = L.Fabric(**config.fabric, loggers=logger)
-            self._fabric_instance.launch()
+                self._fabric_instance = instantiate(config.fabric)
+            if not _is_using_cli():  # if not using cli, launch the fabric
+                self._fabric_instance.launch()
             # Set the log directory in config if it is not already set
             if (
                 self.log_dir is not None
@@ -150,3 +172,27 @@ class LightningFabricMixin:
             return True
         else:
             return False
+
+    def log(self, name: str, value: Any, step: Optional[int] = None):
+        """
+        Logs the metric to the fabric's logger.
+        """
+        self.fabric.log(name, value, step=step)
+
+    def log_dict(self, metrics: dict, step: Optional[int] = None):
+        """
+        Logs the metrics to the fabric's logger.
+        """
+        self.fabric.log_dict(metrics, step=step)
+
+    def log_optimizer_lr(
+        self,
+        optimizer: torch.optim.Optimizer,
+        step: Optional[int] = None,
+        name_template: str = "train/lr_group_{0}",
+    ):
+        """
+        Logs the learning rate of the optimizer to the fabric's logger.
+        """
+        for i, param_group in enumerate(optimizer.param_groups):
+            self.fabric.log(name_template.format(i), param_group["lr"], step=step)
