@@ -1,9 +1,8 @@
 import logging
 
-import peft
 from omegaconf import DictConfig
 from peft import PeftModel
-from transformers import AutoModelForSeq2SeqLM, AutoTokenizer
+from transformers import AutoModelForSeq2SeqLM
 
 from fusion_bench.compat.modelpool.base_pool import ModelPool
 from fusion_bench.utils import timeit_context
